#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['seedbank']

package_data = \
{'': ['*']}

install_requires = \
['numpy >=1.17']

extras_require = \
{'dev': ['flit', 'flake8', 'ipython', 'sphinx-autobuild'],
 'doc': ['sphinx', 'furo'],
 'numba': ['numba'],
 'test': ['pytest >= 6', 'pytest-doctestplus', 'pytest-cov'],
 'tf': ['tensorflow >=2,<3'],
 'torch': ['torch']}

setup(name='seedbank',
      version='0.1.0',
      description='Common infrastructure for initializing random number generators.',
      author='Michael Ekstrand',
      author_email='michaelekstrand@boisestate.edu',
      url='https://seedbank.lensit.org',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>= 3.7',
     )
